/* $Id: clc_ultostr.c,v 1.11 2003/04/28 15:50:22 hfuru Exp $ */
/*
 * Copyright(c) 2003, Jan Engelhardt <hirogen2@gmx.de>
 * COPYING RESTRICTIONS APPLY, see COPYING file.
 */

#include <stddef.h>
#include <string.h>
#include "clc_assert.h"
#include "clc_string.h"


size_t
clc_ultostr(char *dest, size_t size, unsigned long num, int base)
{
	const char *sym = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	char *sp;
	size_t ret;

	clc_assert_arg(clc_ultostr, dest != NULL || size == 0);
	clc_assert_arg(clc_ultostr, base >= 2 && base <= 36);

	sp = dest;

#ifdef CLC_FAST
	if((base & (base-1)) == 0) {
		static const unsigned int log[] = { 1,2,3,0, 4,0,0,0, 5 };
		unsigned int shift = log[base / 4];
		unsigned int mask  = base - 1U;
		if (size) {
			while (--size) {
				*sp++ = sym[(unsigned int)num & mask];
				num >>= shift;
				if (num == 0)
					break;
			}
			size = 1;
		}
		for (ret = sp - dest; num; num >>= shift)
			++ret;
	}
	else
#endif /* CLC_FAST */
	{
		if (size) {
			while (--size) {
				*sp++ = sym[num % base];
				num /= base;
				if (num == 0)
					break;
			}
			size = 1;
		}
		for (ret = sp - dest; num; num /= base)
			++ret;
	}
	if (ret == 0)
		ret = 1;

	if (size) {
		*sp = '\0';
		clc_memrev(dest, sp - dest);
	}
	return ret;
}
